/*  Move News from slip:spool/news to uunews:                           */

/*  Pre-RCS Version history:

    Version 1.3 -   Now it works with 1.3 & 2.0 since only Find is used
                    as an external program. The history file is updated
                    last and held in memory for checking. This is
                    much faster.

    Version 1.4 -   Speeded up by stops locks to various directories by
                    Arexx process when writeln() return was not trapped.
                    Strange! Must be a feature.

    Version 1.5 -   Oooops! Missed a close-comment when deleting a line!

    Version 1.6 -   Changed the size for the rnews line and changed logic
                    so totals only print if total articles GT 0
    
 */

/*  
 *  $Header$
 *   
 *  $Log$
 *
 */

VERSION = "$Header$"


TRUE  = 1  /* Just in case I forget! :-) */
FALSE = 0



/*  Load RexxSupport.library to get Process ID for Work files */

if ~show('L','rexxsupport.library') then do

    if ~addlib('rexxsupport.library',0,-30,0) then do
        say 'Cannot open REXXSupport.library'
        exit 10
        end
    end



/*  The next two files should be on a fast device, RAM: is best but HD
    will do if memory is limited. Remember that the size of this file
    will be the same as the amount of news downloaded, since we process
    it in one pass at the end.                                          */

findfile = 't:ProcNews.lst.' || pragma('I')
workfile = 't:ProcNews.wrk.' || pragma('I')

remlib('rexxsupport.library')



/*  The file containing the previously processed Message-ID's, this
    should be pruned (FIFO fashion) so that ~3 days of ID's are
    kept at any one time.                                               */

history  = 'nntp:history.procnews'



/*  Specify a path as first (only) argument if nntp: does not point to
    your AmigaNOS news tree.                                            */

parse arg path .

if path == '' then path = 'nntp:'



/*  Open (and maybe create the history file.                            */

if ~open( history, history, 'R' ) then do

    say 'Creating History File...'

    if ~open( history, history, 'W' ) then do
        say 'Cannot Open or Create History file (' || history || ')'
        exit 1
        end

    close( history )

    if ~open( history, history, 'R' ) then do
        say 'Unexpected Error Reopening History File'
        exit 1
        end

    end



/*  Read each Message-ID contained in the history file and add to the
    stem variable to use for checking later. A check of a message-id
    against this variable will give 1 if present and ~1 if not.         */

hist_count = 0
addhist_count = 0

do forever

    id = readln( history )

    if eof( history ) then leave

    hist_count = hist_count + 1
    hist.id = 1

    end

close( history )



/*  Finally, we open the workfile which will be read and processed by
    rnews for all those news articles.                                  */

if ~open( workfile, workfile, 'W' ) then do

    say 'Cannot Open Article Workfile (' || workfile || ')'
    exit 1
    end



/*  Run find to search for the .txt files that NOS leaves lying about.  */

address command 'find >' || findfile path '-name *.txt -print'



/*  Reset the article counts                                            */

art_total  = 0
art_ok     = 0
art_reject = 0



/*  Process the findfile file by file                                   */

if ~open( findfile, findfile, 'R' ) then do
    say 'Error Opening Work File. (' || findfile || ')'
    exit 1

    end

do forever

    newsfile = readln(findfile)
    if eof(findfile) then leave

    if ~open( newsfile, newsfile, 'R' ) then do
        iterate
        end

    abort = FALSE

    do forever

        line = readln( newsfile )
        if eof( newsfile ) then leave

        if substr( line, 1, 5 ) == 'From ' then do

            header.1 = line
            count = 2
            abort = FALSE
            messageid = ''

            art_total = art_total + 1

            do forever

                headline = readln( newsfile )
                if eof( newsfile ) then do

                    say '*** Incomplete Article ***'
                    abort = 1
                    leave
                    end

                if headline == '' then do

                    if messageid == '' then leave

                    max_count = count - 1

                    if ~id_process() then do

                        abort = TRUE

                        art_reject = art_reject + 1

                        leave
                        end

                    art_ok = art_ok + 1

                    rcode = writeln( workfile, '#! rnews 200000')

                    do count=1 to max_count

                        rcode = writeln( workfile, header.count)
                        header.count = ''
                        end

                    rcode = writeln( workfile, headline )
                    leave
                    end

                if upper(substr( headline, 1, 12 )) == 'MESSAGE-ID: ' then do
                    messageid = word( headline, 2 )
                    end

                header.count = headline
                count = count + 1
                end

            iterate
            end

        if ~abort then rcode = writeln( workfile, line )

        end

    rcode = close( newsfile )

    address command 'delete >nil:' newsfile

    end


close( findfile )
close( workfile )

address command 'rnews' workfile

if art_total > 0 then address command 'rx logger nntp:nntp.log' workfile

address command 'copy >nil:' workfile 'nntp:procnews.workfile'
address command 'delete >nil:' findfile workfile



/*  Update the history file                                             */

if ~open( history, history, 'A' ) then do

    say 'Unexpected Error appending to History File'
    exit 1
    end


do count=1 to addhist_count

    rcode = writeln( history, addhist.count )
    end


/*  Pointless statistics!                                               */


if art_total > 0 then do
    say 'ProcessNews: Articles Processed: ' art_ok
    say '                      Rejected:  ' art_reject
    say '                      Total:     ' art_total
    end

exit 0




id_process:  /* Returns TRUE if Article is NEW, FALSE if Article OLD    */


if hist.messageid == 1 then return FALSE

hist.messageid = 1
hist_count = hist_count + 1

addhist_count = addhist_count + 1
addhist.addhist_count = messageid

return TRUE

